/* ************************************************************************** */
/*                                                                            */
/*  monochrome.h                                                              */
/*  | Project : libmonochrome                                                 */
/*                                                                            */
/*  By: Pierre "PierrotLL" Le Gall <legallpierre89@gmail.com>                 */
/*  Last updated: 2011/11/22                                                  */
/*                                                                            */
/* ************************************************************************** */

#ifndef MONOCHROME_H
# define MONOCHROME_H
# ifdef __cplusplus
extern "C" {
# endif

# define ML_SCREEN_WIDTH     128
# define ML_SCREEN_HEIGHT    64

# define ML_CONTRAST_MIN     130
# define ML_CONTRAST_NORMAL  168
# define ML_CONTRAST_MAX     190

typedef enum {
	ML_TRANSPARENT = -1,
	ML_WHITE,
	ML_BLACK,
	ML_XOR,
	ML_CHECKER
} ML_Color;

typedef enum {
	ML_MOR,
	ML_MAND,
	ML_MXOR
} ML_Mode;

/*
**	Function prototypes
*/

# define ML_vram_adress() ML_vram_address() // those damn youth mistakes
void* ML_vram_address();

void ML_clear_vram();
void ML_clear_screen();
void ML_display_vram();

void ML_set_contrast(unsigned char contrast);
unsigned char ML_get_contrast();

void ML_pixel(int x, int y, ML_Color color);
void ML_point(int x, int y, int width, ML_Color color);
ML_Color ML_pixel_test(int x, int y);

void ML_line(int x1, int y1, int x2, int y2, ML_Color color);
void ML_horizontal_line(int y, int x1, int x2, ML_Color color);
void ML_vertical_line(int x, int y1, int y2, ML_Color color);

void ML_rectangle(int x1, int y1, int x2, int y2, int border_width, ML_Color border_color, ML_Color fill_color);

void ML_polygon(const int *x, const int *y, int nb_vertices, ML_Color color);
void ML_filled_polygon(const int *x, const int *y, int nb_vertices, ML_Color color);

void ML_circle(int x, int y, int radius, ML_Color color);
void ML_filled_circle(int x, int y, int radius, ML_Color color);

void ML_ellipse(int x, int y, int radius1, int radius2, ML_Color color);
void ML_ellipse_in_rect(int x1, int y1, int x2, int y2, ML_Color color);
void ML_filled_ellipse(int x, int y, int radius1, int radius2, ML_Color color);
void ML_filled_ellipse_in_rect(int x1, int y1, int x2, int y2, ML_Color color);

void ML_horizontal_scroll(int scroll);
void ML_vertical_scroll(int scroll);

void ML_bmp_or(const unsigned char *bmp, int x, int y, int width, int height);
void ML_bmp_and(const unsigned char *bmp, int x, int y, int width, int height);
void ML_bmp_xor(const unsigned char *bmp, int x, int y, int width, int height);
void ML_bmp_or_cl(const unsigned char *bmp, int x, int y, int width, int height);
void ML_bmp_and_cl(const unsigned char *bmp, int x, int y, int width, int height);
void ML_bmp_xor_cl(const unsigned char *bmp, int x, int y, int width, int height);

void ML_bmp_8_or(const unsigned char *bmp, int x, int y);
void ML_bmp_8_and(const unsigned char *bmp, int x, int y);
void ML_bmp_8_xor(const unsigned char *bmp, int x, int y);
void ML_bmp_8_or_cl(const unsigned char *bmp, int x, int y);
void ML_bmp_8_and_cl(const unsigned char *bmp, int x, int y);
void ML_bmp_8_xor_cl(const unsigned char *bmp, int x, int y);

void ML_bmp_16_or(const unsigned short *bmp, int x, int y);
void ML_bmp_16_and(const unsigned short *bmp, int x, int y);
void ML_bmp_16_xor(const unsigned short *bmp, int x, int y);
void ML_bmp_16_or_cl(const unsigned short *bmp, int x, int y);
void ML_bmp_16_and_cl(const unsigned short *bmp, int x, int y);
void ML_bmp_16_xor_cl(const unsigned short *bmp, int x, int y);

#define ML_background_or(B) ML_background((B), ML_OR)
#define ML_background_and(B) ML_background((B), ML_AND)
#define ML_background_xor(B) ML_background((B), ML_XOR)
void ML_background(const void *bmp, ML_Mode mode);

# ifdef __cplusplus
}
# endif
#endif
